/* ------------------------------------------------------------------------------------------------ */
/* FILE :		Demo_3_SMP_ZEDBOARD_A9_XSDK.c														*/
/*																									*/
/* CONTENTS :																						*/
/*				Demo for the ZedBoard (Cortex-A9) development board									*/
/*				Operation of the RTOS shown on the serial port										*/
/*				Tasks info & configuration through the serial port									*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:      1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*							Emulation: VT-100														*/
/*																									*/
/*																									*/
/* Copyright (c) 2012-2014, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.5 $																				*/
/*	$Date: 2012/09/08 16:03:01 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if OS_DEMO != 3
	#error "Wrong OS_DEMO value" 
#endif

#include <ctype.h>
#include "mAbassi.h"
#include "xparameters.h"
#include "xuartps_hw.h"
#include "xgpio.h"

/* ------------------------------------------------ */

#define SYS_FREQ	666666667						/* Processor clock frequency					*/

/* ------------------------------------------------ */

#define TSK_REFRESH_RATE	10						/* Display refresh rate: 1/TSK_REFRESH_RATE sec	*/
#define RTC_REFRESH_RATE	2						/* RTC refresh rate related to TSK_REFRESH_RATE	*/

#define USE_EVENTS			0						/* Events or mailboxes used to inform task 		*/

#define OP_ZERO_CNT			1						/* Operation requested by the ISR state machine	*/
#define OP_LOCK_MTX			2
#define OP_UNLOCK_MTX		4

#define TSK_HIGH			0						/* Index of the task info in TskDsc[]			*/
#define TSK_RRA				1
#define TSK_RRB				2
#define TSK_RRC				3
#define TSK_STRVA			4
#define TSK_STRVB			5

#define OP_ROUND_ROBIN		0						/* Command line configuration operation			*/
#define OP_STARVE_RUN		1
#define OP_STARVE_WAIT		2
#define OP_STARVE_PRIO		3
#define OP_PRIO				4

#define MBX_DISP_SIZE		32						/* Depth of the mailbox for the display			*/

/* ------------------------------------------------ */

typedef struct {									/* Hold the descriptions for the task monitored	*/
	TSK_t *Task;									/* Task descriptor								*/
  #if ((USE_EVENTS) == 0)
	MBX_t *Mailbox;									/* Mailbox when ISR state machine uses mailboxes*/
  #endif
	int    Running;									/* If this is the running task (ignoring A&E	*/
	int    WasReset;								/* To inform task to reset running time counter	*/
	int    CPUcnt;									/* Number of timer ticks task has been running	*/
	int    WillBlk;									/* If the task is next to block on the mutex	*/
	int    WillMtx;									/* If the task is next to lock the mutex		*/
} TskDsc_t;
TskDsc_t TskDsc[6];

typedef struct {									/* Info sent to the task to update the display	*/
	char *Str;										/* String to print on the display				*/
	int  X;											/* X coordinate									*/
	int  Y;											/* Y coordinate									*/
} DispInfo_t;
DispInfo_t DispInfo[31];

volatile int g_DispIdx=0;							/* Entry in DispInfo[] is next to be filled		*/
volatile int g_DispLock;							/* Mutual exclusion access to printing			*/

char Display[7][20] = { "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
						"00:00:00   "
                      };
volatile int g_CmdIdx;								/* Index where to write next character from kyb	*/
TSK_t *AdamEve;
MTX_t *Mutex;										/* Mutex used by the task to show inheritance	*/
MBX_t *MbxDisp;										/* Mailbox to send display updates to the tasks	*/
SEM_t *SemaAE;										/* Semaphore to awake Adam & Eve				*/

#if ((OX_STARVE_RUN_MAX) != 0)
  extern TSK_t *g_StrvList;							/* Imported from mAbassi.c						*/
#else
  TSK_t *g_StrvList = (TSK_t *)NULL;
#endif

#if ((OX_SEARCH_ALGO) >= 0)							/* Different way to get task list				*/
  extern TSK_t *g_TaskRdy[(OX_PRIO_MIN)+3];			/* Imported from mAbassi.c						*/
#else
  extern TSK_t *g_PrioList;							/* Imported from mAbassi.c						*/
#endif

XGpio G_GpioLed;									/* LEDs gpio descriptor							*/

/* ------------------------------------------------ */

void   Fct(void);									/* A single function implements all the tasks	*/
int    GetKey(void);								/* Non-blocking keyboard input					*/
void   TIMERinit(unsigned int Reload);
void   UpdateDisp(DispInfo_t *Disp);

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

int main(void)
{
int   ccc;											/* Character read from the keyboard				*/
int   Done;											/* If command decoding is done					*/
int   Error;										/* If error during command decoding				*/
int   ii;											/* General purpose								*/
int   jj;											/* General purpose								*/
int   kk;											/* General purpose								*/
int   Op;											/* Operation requested by the command			*/
int   RdIdx;										/* Read index in the command line string		*/
int   Row;											/* Row of the display where to print the sting	*/
char *String;										/* Display sting being created					*/
int   TaskIdx;										/* Index of the task specified on command line	*/
int   Toggle;										/* Toggle use to control refresh of RTC			*/

static char Cmd[64];								/* Command line from keyboard					*/

/* ------------------------------------------------ */
/* UART set-up										*/

	setvbuf(stdin,  NULL, _IONBF, 0);				/* By default, NewLib library flush the I/O		*/
	setvbuf(stdout, NULL, _IONBF, 0);				/* buffer when full or when new-line			*/

	puts("System started");

/* ------------------------------------------------ */
/* 	Start the RTOS									*/

	OSstart();

	OSeint(1);

	AdamEve = TSKmyID();

/* ------------------------------------------------ */
/* Application set-up								*/
													/* Need to create all services before allowing	*/
	MbxDisp                  = MBXopen("Disp", MBX_DISP_SIZE);
	Mutex                    = MTXopen("Mutex");
	SemaAE                   = SEMopen("Sem A&E"); 

	TskDsc[TSK_HIGH].Task    = TSKcreate("HI   ", 2, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_RRA].Task     = TSKcreate("RRA  ", 4, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_RRB].Task     = TSKcreate("RRB  ", 4, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_RRC].Task     = TSKcreate("RRC  ", 4, TC_TSK_STACK, &Fct, 0);

	TskDsc[TSK_STRVA].Task   = TSKcreate("STRVA", 7, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_STRVB].Task   = TSKcreate("STRVB", 9, TC_TSK_STACK, &Fct, 0);

  #if ((OX_ROUND_ROBIN) < 0)
	TSKsetRR(TskDsc[TSK_RRA].Task, 4*OX_RR_TICK);
	TSKsetRR(TskDsc[TSK_RRB].Task, 6*OX_RR_TICK);
	TSKsetRR(TskDsc[TSK_RRC].Task, 8*OX_RR_TICK);
  #endif

  #if ((OX_STARVE_RUN_MAX) < 0)
	TSKsetStrvRunMax (TskDsc[TSK_STRVA].Task,   5*OS_TICK_PER_SEC);
	TSKsetStrvRunMax (TskDsc[TSK_STRVB].Task,   5*OS_TICK_PER_SEC);
  #endif

  #if ((OX_STARVE_WAIT_MAX) < 0)
	TSKsetStrvWaitMax(TskDsc[TSK_STRVA].Task, (25*OS_TICK_PER_SEC)/10);
	TSKsetStrvWaitMax(TskDsc[TSK_STRVB].Task, (10*OS_TICK_PER_SEC)/10);
  #endif

  #if ((OX_MP_TYPE) & 4U) && ((OX_N_CORE) != 1)
	TSKsetCore(TskDsc[TSK_HIGH].Task, 1);
	TSKsetCore(TskDsc[TSK_RRA].Task,  1);
	TSKsetCore(TskDsc[TSK_RRB].Task,  1);
  #endif

	for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {
		TSKsetArg(TskDsc[ii].Task, &TskDsc[ii]);
	  #if ((USE_EVENTS) == 0)
		TskDsc[ii].Mailbox  = MBXopen(NULL, 32);
	  #endif
		TskDsc[ii].CPUcnt   = 0;
		TskDsc[ii].Running  = 0;
		TskDsc[ii].WasReset = 0;
		TskDsc[ii].WillBlk  = 0;
		TskDsc[ii].WillMtx  = 0;
		TSKresume(TskDsc[ii].Task);
	}

	printf("\033[H\033[J");

	puts("\n     mAbassi Demo\n");
	puts("High  00.0s  P2     ");
	puts("RR-A  00.0s  P4     ");
	puts("RR-B  00.0s  P4     ");
	puts("RR-C  00.0s  P4     ");
	puts("ST-A  00.0s  P7     ");
	puts("ST-B  00.0s  P9     ");

	printf("\033[15;0H");							/* Set cursor where desired						*/
	printf("mAbassi Demo> ");

/* ------------------------------------------------ */
/* LEDs Set-up										*/
								
	XGpio_Initialize(&G_GpioLed, XPAR_LED_DEVICE_ID);
	XGpio_SetDataDirection(&G_GpioLed, 1, 0);
	XGpio_DiscreteWrite(&G_GpioLed, 1, 0);			/* Turn off all the LEDs						*/

/* ------------------------------------------------ */
/* SysTick Set-up									*/
								
	OSisrInstall(29, &TIMtick);						/* Install the RTOS timer tick function			*/
	TIMERinit(((((SYS_FREQ/2)/100000)*(OS_TIMER_US))/(10)));	/* Zynq set with a /2 prescaler		*/
	GICenable(29, 128, 1);							/* Private timer interrupt ID is 29 prio=mid	*/

	g_CmdIdx   = 0;
	g_DispLock = 0;
	Toggle     = 0;

	for (;;) {

		SEMwait(SemaAE, -1);

		if (++Toggle >= RTC_REFRESH_RATE) { 		/* RTC is not always refreshed					*/
			Toggle = 0;
			String = &Display[6][0];				/* Display line to update						*/
													/* RTC display is up to 99:59:59 so				*/
			jj = OSdint();							/* roll back 00:00:00 when max reached			*/
			if (G_OStimCnt >= (360000L*(long)OS_TICK_PER_SEC)) {
				G_OStimCnt -= (360000L*(long)OS_TICK_PER_SEC);
			}
			OSeint(jj);
			jj = G_OStimCnt;

			kk = jj/(OS_TICK_PER_SEC*3600*10);		/* MS digit for the hours						*/
			String[0] = '0'+kk;
			jj -= kk*(OS_TICK_PER_SEC*3600*10);

			kk  =jj/(OS_TICK_PER_SEC*3600);			/* LS digits for the hours						*/
			String[1] = '0'+kk;
			jj -= kk*(OS_TICK_PER_SEC*3600);

			kk  = jj/(OS_TICK_PER_SEC*600);			/* MS digits for the minutes					*/
			String[3] = '0'+kk;
			jj -= kk*(OS_TICK_PER_SEC*600);

			kk  = jj/(OS_TICK_PER_SEC*60);			/* LS digits for the minutes					*/
			String[4] = '0'+kk;
			jj -= kk*(OS_TICK_PER_SEC*60);

			kk  = jj/(OS_TICK_PER_SEC*10);			/* MS digits for the seconds					*/
			String[6] = '0'+kk;
			jj -= kk*(OS_TICK_PER_SEC*10);

			kk = jj/(OS_TICK_PER_SEC);				/* LS digits for the seconds					*/
			String[7] = '0'+kk;

			Row = 11;
			DispInfo[g_DispIdx].Str = String;
			DispInfo[g_DispIdx].X   = Row;			/* Send the string to the running task			*/
			DispInfo[g_DispIdx].Y   = 7;

			MBXput(MbxDisp, (intptr_t)&DispInfo[g_DispIdx], 0);
			if (++g_DispIdx >= (sizeof(DispInfo)/sizeof(DispInfo[0]))) {
				g_DispIdx = 0;
			}
		}

		for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {	/* Update all tasks status		*/
			String = &Display[ii][0];				/* Display line to update						*/
			jj = (TskDsc[ii].CPUcnt*100)/(OS_TICK_PER_SEC);
			if (jj > 9999) {						/* Can't display more than 99.9 (is /10)		*/
				jj = 9999;
			}
			kk = jj/1000;							/* Running time MS digit						*/
			String[0] = '0'+kk;

			jj = jj - (kk*1000);					/* Running time LS digit						*/
			kk = jj/100;
			String[1] = '0'+kk;

			jj = jj - (kk*100);						/* Running time fraction						*/
			kk = jj/10;
			String[3] = '0'+kk;

			if (TSKisSusp(TskDsc[ii].Task)) {		/* If the task is suspended, the priority		*/
				String[7] = 'S';					/* is replaced by "S "							*/
				String[8] = ' ';
			}
			else {									/* Regular priority field						*/
				String[7] = 'P';
				if (TskDsc[ii].Task->SuspRqst != 0) {
					String[8] = 'S';				/* When the task will be suspended, indicate so	*/
				}
				else {								/* Priority numerical value						*/
					String[8] = '0'+TSKgetPrio(TskDsc[ii].Task);
				}
			}
			if (TskDsc[ii].Running != 0) {			/* If this is a running task, show the core #	*/
			  #if ((OX_N_CORE) == 1) || (((OX_MP_TYPE)&6U) == 0)
				String[11] = 'R';					/* 'R' instead of Core #0 when single core		*/
			  #else
				String[11] = '0'+(TskDsc[ii].Running-1);
			  #endif
			}
			else {									/* Not a running task, nothing in this field	*/
				String[11] = ' ';
			}
			if (TskDsc[ii].Task == g_StrvList) {	/* If the task is under starvation protection	*/
				String[12] = 'A';					/* Show the "A"									*/
			}
			else {									/* Not under starvation protection, nothing to	*/
				String[12] = ' ';					/* shown in this field							*/
			}

			if (TskDsc[ii].Task->MyMutex == Mutex) {
				String[13] = 'M';					/* If this task locks the mutex, show it		*/
			}
			else if (TskDsc[ii].WillMtx  != 0) {
				String[13] = 'm';					/* If the task will lock the mutex, show it		*/
			}
			else if (TskDsc[ii].Task->Blocker == Mutex) {
				String[13] = 'B';					/* If the task is blocked by the mutex, show it	*/
			}
			else if (TskDsc[ii].WillBlk  != 0) {
				String[13] = 'b';					/* if the task will block on the mutex, show it	*/
			}
			else {
				String[13] = ' ';					/* If the task not involved with the mutex, 	*/
			}										/* nothing to show in this field				*/

			if ((TSKgetPrio(TskDsc[ii].Task) != TskDsc[ii].Task->OriPrio)
			&&  (TskDsc[ii].Task != g_StrvList)
			&&  (TskDsc[ii].Task->MyMutex == Mutex)) {
				String[14] = 'I';					/* If the task has it's priority increased due	*/
			}										/* to priority inheritance, show this			*/
			else {
				String[14] = ' ';
			}
			Row = 4+ii;
			DispInfo[g_DispIdx].Str = String;
			DispInfo[g_DispIdx].X   = Row;
			DispInfo[g_DispIdx].Y   = 7;
													/* Send the string to display to running task	*/
			MBXput(MbxDisp, (intptr_t)&DispInfo[g_DispIdx], 0);
			if (++g_DispIdx >= (sizeof(DispInfo)/sizeof(DispInfo[0]))) {
				g_DispIdx = 0;
			}
		}

		ccc = GetKey();								/* Check for a new key pressed on the keyboard 	*/
		if (ccc != 0) {								/* When A&E is printing, don't clash with other	*/
			CORElock(4, &g_DispLock, 0, COREgetID()+1);	/* tasks.  No need for OSdint() / OSeint()	*/

			if (g_CmdIdx < (sizeof(Cmd)/sizeof(Cmd[0]))-2) {
				putchar(ccc);						/* A key was pressed , echo it					*/
			}										/* If command line buffer full, don't echo		*/
			if (ccc == '\r') {						/* Need to remap CR to '\n'						*/
				putchar('\n');
			}
			else if (ccc == '\b') {					/* Backspace pressed, to erase previous key		*/
				if (g_CmdIdx != 0) {
					g_CmdIdx--;
					putchar(' ');
					putchar('\b');
				}
			}
			else {
				if (g_CmdIdx == 0) {				/* Add to the command line without overflow		*/
					printf("\033[15;16H");
					for (ii=1 ; ii<sizeof(Cmd) ; ii++) {
						putchar(' ');
					}
					printf("\033[15;16H");
				}
				if (g_CmdIdx < (sizeof(Cmd)/sizeof(Cmd[0]))-2) {
					Cmd[g_CmdIdx] = toupper(ccc);
					g_CmdIdx++;
				}
			}

			if (ccc == '\r') {						/* CR pressed, then process the command			*/
				Cmd[g_CmdIdx] = '\0';
				TaskIdx = -1;
				RdIdx   = 0;
				while ((Cmd[RdIdx] == ' ')			/* Skip start of line white spaces				*/
				||     (Cmd[RdIdx] == ',')
				||     (Cmd[RdIdx] == '\t')) {
					RdIdx++;
				}

				Done  = 0;
				Error = 0;

				if (g_CmdIdx == RdIdx) {			/* When everything white space, nothing to do	*/
					Done = 1;
				}

				g_CmdIdx = 0;

				printf("\033[15;0H");

				if ((0 == strncmp(&Cmd[RdIdx], "?", 1))
				||  (0 == strncmp(&Cmd[RdIdx], "INFO", 4))) {
					putchar('\n');
					putchar('\n');
					for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {
						printf("%s  -->  ",  TskDsc[ii].Task->TskName);
					  #if ((OX_ROUND_ROBIN) != 0)
						printf("RR %3d  |  ", TSKgetRR(TskDsc[ii].Task));
					  #endif
					  #if ((OX_STARVE_RUN_MAX) != 0)
						printf("SP %1d  |  ", TSKgetStrvPrio(TskDsc[ii].Task));
						printf("SR %3d  |  ", TSKgetStrvRunMax(TskDsc[ii].Task));
						printf("SW %3d  |  ", TSKgetStrvWaitMax(TskDsc[ii].Task));
					  #endif
						printf("P %d      \n",  TskDsc[ii].Task->OriPrio);
					}
					putchar('\n');
					Done = 1;
					printf("\033[15;15H       ");
				}

				if (0 == strncmp(&Cmd[RdIdx], "HI", 2)) {
					TaskIdx = TSK_HIGH;
					RdIdx += 2;
				}
				if (0 == strncmp(&Cmd[RdIdx], "RRA", 3)) {
					TaskIdx = TSK_RRA;
					RdIdx += 3;
				}
				if (0 == strncmp(&Cmd[RdIdx], "RRB", 3)) {
					TaskIdx = TSK_RRB;
					RdIdx += 3;
				}
				if (0 == strncmp(&Cmd[RdIdx], "RRC", 3)) {
					TaskIdx = TSK_RRC;
					RdIdx += 3;
				}
				if (0 == strncmp(&Cmd[RdIdx], "STRVA", 5)) {
					TaskIdx = TSK_STRVA;
					RdIdx += 5;
				}
				if (0 == strncmp(&Cmd[RdIdx], "STRVB", 5)) {
					TaskIdx = TSK_STRVB;
					RdIdx += 5;
				}
				if (Done == 0) {
					if (TaskIdx < 0) {
						printf("\033[15;15H");		/* Set cursor after mAbassi Demo>				*/
						ii = printf("Unrecognized task name");
						for( ; ii<sizeof(Cmd) ; ii++) {
							putchar(' ');
						}
						Error=1;
					}
				}

				Op = -1;							/* 2nd token if the service, check all			*/
				if ((Error == 0)
				&&  (Done == 0)) {
					while ((Cmd[RdIdx] == ' ')		/* Skip white spaces between 1st and 2nd token	*/
					||     (Cmd[RdIdx] == ',')
					||     (Cmd[RdIdx] == '\t')) {
						RdIdx++;
					}
					if (0 == strncmp(&Cmd[RdIdx], "RR", 2)) {
						Op = OP_ROUND_ROBIN;		/* Token names are re-mapped to operations #	*/
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "SR", 2)) {
						Op = OP_STARVE_RUN;
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "SP", 2)) {
						Op = OP_STARVE_PRIO;
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "SW", 2)) {
						Op = OP_STARVE_WAIT;
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "P", 1)) {
						Op = OP_PRIO;
						RdIdx += 1;
					}
					if (Op < 0) {
						Error = 1;
						printf("\033[15;15H");		/* Set cursor after mAbassi Demo>				*/
						ii = printf("Unrecognized property        ");
						for( ; ii<sizeof(Cmd) ; ii++) {
							putchar(' ');
						}
					}
				}

				if ((Error == 0)
				&&  (Done == 0)) {
					while ((Cmd[RdIdx] == ' ')		/* Skip white spaces between 2nd and 3rd token	*/
					||     (Cmd[RdIdx] == ',')
					||     (Cmd[RdIdx] == '\t')) {
						RdIdx++;
					}
					ii = 0;
					do {							/* 3rd token is the service numerical value		*/
						if ((Cmd[RdIdx] >= '0')
						&&  (Cmd[RdIdx] <= '9')) {
							ii = ii*10;
							ii += Cmd[RdIdx] - '0';
							RdIdx ++;
						}
						else if (Cmd[RdIdx] == '\0') {
							Done = 1;
						}
						else {
							Error = 1;
							printf("\033[15;15H");	/* Set cursor after mAbassi Demo>				*/
							ii = printf("Invalid number");
							for( ; ii<sizeof(Cmd) ; ii++) {
								putchar(' ');
							}
						}
					} while ((Error == 0)
					  &&     (Done  == 0));

					if (Error == 0) {				/* When everything OK, set the numerical value	*/
						switch(Op) {				/* to the service in the task					*/
						case OP_ROUND_ROBIN:
						  #if ((OX_ROUND_ROBIN) < 0)
							TSKsetRR(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						case OP_STARVE_PRIO:
						  #if ((OX_STARVE_PRIO) < 0)
							TSKsetStrvPrio(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						case OP_STARVE_RUN:
						  #if ((OX_STARVE_RUN_MAX) < 0)
							TSKsetStrvRunMax(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						case OP_STARVE_WAIT:
						  #if ((OX_STARVE_WAIT_MAX) < 0)
							TSKsetStrvWaitMax(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						default:
							TSKsetPrio(TskDsc[TaskIdx].Task, ii);
							break;
						}
					}
				}
				printf("\033[15;0H");				/* Set cursor at beginning of line				*/
				printf("mAbassi Demo> ");
			}
			COREunlock(4, &g_DispLock, 0);
		}
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Idle Task																						*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void IdleTask(void)
{
DispInfo_t *Disp;
int         ii;
intptr_t    Msg;

 #if defined(OS_NEWLIB_REENT)						/* In case the reentrance is set on per task	*/
  #if ((OS_NEWLIB_REENT) > 0)
	setvbuf(stdin,  NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);
  #endif
 #endif

	MbxDisp = MBXopen("Disp", MBX_DISP_SIZE);		/* Example of run time safe creation			*/

	for(;;) {
		ii = OSdint();								/* Will print, don't want to be pre-empted		*/
		CORElock(4, &g_DispLock, 0, COREgetID()+1);	/* Done outside if() to not have pending write	*/
		if (0 ==  MBXget(MbxDisp, &Msg, 0)) {		/* Check if A&E has sent a string to display	*/
			Disp = (void *)Msg;
			UpdateDisp(Disp);
		}
		COREunlock(4, &g_DispLock, 0);
		OSeint(ii);
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Tasks function																					*/
/*																									*/
/* All tasks use the same function.																	*/
/* The Task Argument is the custom task description data structure									*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void Fct(void)
{
DispInfo_t *Disp;
int         ii;
intptr_t    Msg;
int         NoNewMsg;
TskDsc_t   *MyInfo;

 #if defined(OS_NEWLIB_REENT)						/* In case the reentrance is set on per task	*/
  #if ((OS_NEWLIB_REENT) > 0)
	setvbuf(stdin,  NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);
  #endif
 #endif

	MyInfo = (TskDsc_t *)TSKgetArg();				/* Get the argument for this task				*/

	for ( ; ; ) {
	  #if USE_EVENTS								/* Info from ISR state machine comes from 		*/
		NoNewMsg = EVTwait(0x0000, 0xFFFF, 0);		/* either an event flag or a mailbox			*/
		Msg      = EVTget();
	  #else
		NoNewMsg = MBXget(MyInfo->Mailbox, &Msg, 0);
	  #endif

		if (NoNewMsg == 0) {						/* When a valid sync trigger was received		*/

			if ((int)Msg & OP_ZERO_CNT) {			/* Request to zero the running time				*/
				MyInfo->CPUcnt   = 0;
				MyInfo->WasReset = 0;
			}
			if (((int)Msg & OP_LOCK_MTX)			/* Request from ISR to lock the mutex			*/
			&&  (MTXowner(Mutex) != TSKmyID())) {	/* Safety in case state machine has issues		*/
				MyInfo->WillBlk = 0;
				MyInfo->WillMtx = 0;
				MTXlock(Mutex, -1);
			}
			if (((int)Msg & OP_UNLOCK_MTX)			/* Request from ISR to unlock the mutex			*/
			&&  (MTXowner(Mutex) == TSKmyID())) {	/* Safety in case state machine has issues		*/
				MTXunlock(Mutex);
			}
		}
		ii = OSdint();								/* Will print, don't want to be pre-empted		*/
		CORElock(4, &g_DispLock, 0, COREgetID()+1);	/* Done outside if() to not have pending write	*/
		if (0 ==  MBXget(MbxDisp, &Msg, 0)) {		/* Check if A&E has sent a string to display	*/
			Disp = (void *)Msg;
			UpdateDisp(Disp);
		}
		COREunlock(4, &g_DispLock, 0);
		OSeint(ii);
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* RTOS Timer call back function																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */
#if USE_EVENTS
	#define SEND_INFO(_Idx, _Info)	EVTset(TskDsc[_Idx].Task, _Info);
#else
	#define SEND_INFO(_Idx, _Info)	MBXput(TskDsc[_Idx].Mailbox, _Info, 0)
#endif

void TIMcallBack(void)
{
int    AErunning;
int    ii;
int    jj;
int    Leds;
int    RunIdx[OX_N_CORE];
static int Cnt       =  0;
static int MtxCnt    =  0;
static int MtxOwner  =  5;
static int MtxSelect =  5;
static int MtxBlk    =  1;
static int MtxState  =  0;
static int SuspCnt   =  0;

	for (ii=0 ; ii<OX_N_CORE ; ii++) {				/* To remove "uninitialized warning"			*/
		RunIdx[ii] = 0;
	}
	Cnt++;											/* Check if A&E is running as we can't update	*/
	AErunning = 0;									/* everything when it does						*/
	for (ii=0 ; ii<OX_N_CORE ; ii++) {
		if (G_OStaskNow[ii] == AdamEve) {
			AErunning = 1;
		}
	}
													/* Update the CPU of the tasks					*/
	if (AErunning == 0) {							/* When A&E is running, don't update			*/
		for (ii=0 ; ii<OX_N_CORE ; ii++) {			/* Invalidate running status of all tasks		*/
			RunIdx[ii] = -1;						/* RunIdx holds the running task indexes		*/
		}
		Leds=0;
		for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {
			TskDsc[ii].Running = 0;
			for (jj=0 ; jj<OX_N_CORE ; jj++) {		/* Check if is one of the current one running	*/
				if (G_OStaskNow[jj] == TskDsc[ii].Task) {
					TskDsc[ii].Running  = jj+1;		/* +1 to show the core #						*/
					RunIdx[jj] = ii;
					Leds |= 1<<ii;
				}
			}
			if (TskDsc[ii].Task->Blocker != (SEM_t *)NULL) {
				if (TskDsc[ii].WasReset == 0) {
					SEND_INFO(ii, OP_ZERO_CNT);
					TskDsc[ii].WasReset = 1;
				}
			}
		 #if ((OX_ROUND_ROBIN) != 0)
		  #if ((OX_ROUND_ROBIN) < 0)
			if (TskDsc[ii].Task->RRmax != 0U)
		  #endif
			 {
				if (TskDsc[ii].Task->RRcnt <= 1U) {
					if (TskDsc[ii].WasReset == 0) {
						SEND_INFO(ii, OP_ZERO_CNT);
						TskDsc[ii].WasReset = 1;
					}
				}
			}
		  #endif
		}
		XGpio_DiscreteWrite(&G_GpioLed, 1, Leds);
	}

	for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {	/* Excluding A&E & Idles			*/
		if (TskDsc[ii].Running != 0) {				/* It is supposed to be running, check if so	*/
			TskDsc[ii].CPUcnt++;
		}
	}

	if (AErunning == 0) {
		switch(MtxState) {
		case 0:										/* State #0 : new mutex owner					*/
			if (++MtxSelect >= sizeof(TskDsc)/sizeof(TskDsc[0])) {
				MtxSelect = 0;
			}
			if (MtxSelect == 2) {					/* RRB if not the first locker of the mutex		*/
				MtxSelect = 4;
			}

			MtxOwner = MtxSelect;

			SEND_INFO(MtxOwner, OP_LOCK_MTX);		/* Tell new owner to lock the mutex				*/

			if (++MtxBlk >= 4) {					/* Select the task that will block				*/
				MtxBlk = 0;							/* Only Hi & RR can block on the mutex			*/
			}

			if (MtxBlk == MtxOwner) {				/* Get another task to block on the mutex		*/
				if (++MtxBlk >= 4) {
					MtxBlk = 0;
				}
			}
			TskDsc[MtxOwner].WillMtx = 1;			/* Display it will lock the mutex soon			*/
			TskDsc[MtxBlk].WillBlk = 1;				/* And this will be blocked 					*/
			MtxState++;
			break;
		case 1:										/* Waiting for the owner to lock the mutex		*/
			if (TskDsc[MtxOwner].Task->MyMutex == Mutex) {
				MtxState++;
			}
			break;
		case 2:										/* OK, the mutex owner is locking the mutex		*/
			SEND_INFO(MtxBlk, OP_LOCK_MTX);			/* Tell the task to be blocked to lock it		*/
			MtxState++;
			break;
		case 3:										/* Make sure the task gets blocked				*/
		  #if ((OX_MP_TYPE & 1) == 0)				/* To be blocked may not run if non-packed		*/
			if (TskDsc[MtxOwner].Task->Prio == TskDsc[MtxBlk].Task->Prio) {
				if (TSKisBlk(TskDsc[MtxBlk].Task)) {/* OK, the task is blocked on the mutex now		*/
					MtxState++;						/* This may take a while as it needs to run		*/
					MtxCnt = 0;
				}
			}
			else {									/* If to be suspended, must skip a few states	*/
				if (TskDsc[MtxOwner].Task->SuspRqst != 0) {	/* as != prio can't run at same time	*/
					MtxState = 4;
					MtxCnt   = 0;
					TskDsc[MtxOwner].WillMtx = 0;
					TskDsc[MtxBlk].WillBlk   = 0;
					TskDsc[MtxBlk].WillMtx   = 1;
				}
			}
		  #elif ((OX_MP_TYPE) == 5)
			if ((TskDsc[MtxOwner].Task->TargetCore == TskDsc[MtxBlk].Task->TargetCore)
			&&  (TskDsc[MtxOwner].Task->TargetCore >= 0)) {
				if (TskDsc[MtxOwner].Task->SuspRqst != 0) {
					MtxState = 4;
					MtxCnt   = 0;
					TskDsc[MtxOwner].WillMtx = 0;
					TskDsc[MtxBlk].WillBlk   = 0;
					TskDsc[MtxBlk].WillMtx   = 1;
				}
			}
			else {									/* If to be suspended, must skip a few states	*/
				if (TSKisBlk(TskDsc[MtxBlk].Task)) {/* OK, the task is blocked on the mutex now		*/
					MtxState++;						/* This may take a while as it needs to run		*/
					MtxCnt = 0;
				}
			}
		  #else
			if (TSKisBlk(TskDsc[MtxBlk].Task)) {	/* OK, the task is blocked on the mutex now		*/
				MtxState++;							/* This may take a while as it needs to run		*/
				MtxCnt = 0;
			}
		  #endif
			break;
		case 4:										/* State #4: Task is blocked on the mutex		*/
			for (ii=0 ; ii<OX_N_CORE ; ii++) {
				if (MtxOwner == RunIdx[ii]) {		/* Make sure the owner is running long enough	*/
					jj = (MtxOwner == 0)			/* When HI task, is getting suspended			*/
					   ? (((15*OS_TICK_PER_SEC)+1) - TskDsc[0].CPUcnt)
					   :   ((5*OS_TICK_PER_SEC)+1);
					if (++MtxCnt > jj) {			/* Tell the owner to release the mutex			*/
						SEND_INFO(MtxOwner, OP_UNLOCK_MTX);
						MtxState++;
					}
				}
			}
			break;
		case 5:										/* Wait for owner to unlock the mutex			*/
			if (TskDsc[MtxOwner].Task->MyMutex != Mutex) {
				MtxOwner = MtxBlk;					/* The new owner is the one that was blocked	*/
				MtxState++;
			}
			break;
		case 6:										/* Wait for new owner to lock the mutex			*/
			if (TskDsc[MtxOwner].Task->MyMutex == Mutex) {
				MtxState++;
				MtxCnt = 0;
			}
			break;
		case 7:
			for (ii=0 ; ii<OX_N_CORE ; ii++) {
				if (MtxOwner == RunIdx[ii]) {		/* Wait a while before telling to release		*/
					jj = (MtxOwner == 0)			/* When HI task, is getting suspended			*/
					   ? (((15*OS_TICK_PER_SEC)+1) - TskDsc[0].CPUcnt)
					   :   ((5*OS_TICK_PER_SEC)+1);
					if (++MtxCnt > jj) {			/* Tell the owner to release the mutex			*/
						SEND_INFO(MtxOwner, OP_UNLOCK_MTX);
						MtxState++;
					}
				}
			}
			break;
		default:									/* Wait for blocked to unlock					*/
			if (TskDsc[MtxOwner].Task->MyMutex != Mutex) {
				MtxState =  0;
				MtxOwner = -1;
			}
			break;
  		}

		if (Cnt >= OS_TICK_PER_SEC/TSK_REFRESH_RATE) {
			SEMpost(SemaAE);						/* Tell A & E to refresh the display			*/
			Cnt = 0;
		}
	}

	SuspCnt++;
	if (SuspCnt == 10*OS_TICK_PER_SEC+3) {			/* Suspend / resume the high priority			*/
		TSKsuspend(TskDsc[TSK_HIGH].Task);
	}
	else if (SuspCnt > 30*OS_TICK_PER_SEC+3) {
		TSKresume(TskDsc[TSK_HIGH].Task);
		SuspCnt = 0;
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void UpdateDisp(DispInfo_t *Disp)
{
int jj;

	if ((Disp->X == 0)								/* Cursor at top of screen						*/
	&&  (Disp->Y == 0)) {
//		printf("\033[H");
		putchar(0x1B);
		putchar('[');
		putchar('H');
	}
	else {
//		printf("\033[%d;%dH%s", Disp->X, Disp->Y, Disp->Str); 
		putchar(0x1B);
		putchar('[');
		if (Disp->X < 10) {
			putchar('0'+Disp->X);
		}
		else {
			putchar('0'+(Disp->X/10));
			putchar('0'+(Disp->X%10));
		}
		putchar(';');
		if (Disp->Y < 10) {
			putchar('0'+Disp->Y);
		}
		else {
			putchar('0'+(Disp->Y/10));			putchar('0'+(Disp->Y%10));
		}
		putchar('H');

		for (jj=0 ; Disp->Str[jj] != '\0' ; jj++){
			putchar(Disp->Str[jj]);
		}

//		printf("\033[15;%dH", g_CmdIdx+15);			/* Set cursor next typing location				*/
		putchar(0x1B);
		putchar('[');
		putchar('1');
		putchar('5');
		putchar(';');
		jj = g_CmdIdx+15;
		if (jj < 10) {
			putchar('0'+jj);
		}
		else {
			putchar('0'+(jj/10));
			putchar('0'+(jj%10));
		}
		putchar('H');
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* Private timer set-up																				*/
/* ------------------------------------------------------------------------------------------------ */

void TIMERinit(unsigned int Reload)
{

volatile unsigned int *TimReg;

	TimReg    = COREgetPeriph();					/* Private timer is at base of Peripherals		*/
	TimReg   += 0x600/sizeof(unsigned int);
	TimReg[2] = 0;									/* Disable timer & interrupts					*/
	TimReg[0] = Reload;								/* Private timer load value						*/
	TimReg[1] = Reload;								/* Start the count at reload value				*/
													/* Private timer control register				*/
	TimReg[2] = (0<<8)								/* Prescaler = /1								*/
	          | (1<<2)								/* Interrupt enable								*/
	          | (1<<1)								/* Auto-reload									*/
	          | (1<<0);								/* Timer enable									*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
{
int RetVal;

	RetVal = 0;
	if (XUartPs_IsReceiveData(STDIN_BASEADDRESS)) {
		RetVal = XUartPs_RecvByte(STDIN_BASEADDRESS);
	}
	return(RetVal);
}

/* EOF */
